@@ -77,7 +77,7 @@ ORBit_alloc_2(size_t block_size,
 		g_malloc(block_size + sizeof(ORBit_mem_info) + before_size)
 		+ before_size);
 
-#ifdef DEBUG
+#ifdef ORBIT_DEBUG
 	block->magic = 0xdeadbeef;
 #endif
 	block->free = freefunc;
@@ -110,7 +110,7 @@ ORBit_free(gpointer mem, CORBA_boolean free_strings)
 
 	block = PTR_TO_MEMINFO(mem);
 
-#ifdef DEBUG
+#ifdef ORBIT_DEBUG
 	g_assert(block->magic == 0xdeadbeef);
 #endif
 
@@ -124,7 +124,7 @@ ORBit_free(gpointer mem, CORBA_boolean free_strings)
 		else
 			my_data = NULL;
 
-#ifdef DEBUG
+#ifdef ORBIT_DEBUG
 		if(block->func_data == NULL)
 			g_warning("block with freefunc %p has no items", block->free);
 #endif
@@ -187,14 +187,21 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 		retval = mem;
 		break;
 	case CORBA_tk_union:
-		subtc = (CORBA_TypeCode)CORBA_Object_duplicate((CORBA_Object)ORBit_get_union_tag(tc, mem), NULL);
-		mem = ((guchar *)mem) + ORBit_gather_alloc_info(tc->discriminator);
-		n = 1;
-		for(i = 0; i < tc->sub_parts; i++) {
-			n = MAX(n, ORBit_find_alignment(tc->subtypes[i]));
+		subtc = (CORBA_TypeCode)CORBA_Object_duplicate(
+		  (CORBA_Object)ORBit_get_union_tag(tc, &mem, TRUE), NULL);
+		{
+		    int sz = 0;
+		    int al = 1;
+		    for(i = 0; i < tc->sub_parts; i++) {
+		        al = MAX(al, ORBit_find_alignment(tc->subtypes[i]));
+		        sz = MAX(sz, ORBit_gather_alloc_info(tc->subtypes[i]));
+		    }
+		    mem = ALIGN_ADDRESS(mem, al);
+		    ORBit_free_via_TypeCode(mem, &subtc, free_strings);
+		    /* the end of the body (subtc) may not be the
+		     * same as the end of the union */
+		    retval = mem + sz;
 		}
-		mem = ALIGN_ADDRESS(mem, n);
-		retval = ORBit_free_via_TypeCode(mem, &subtc, free_strings);
 		break;
 	case CORBA_tk_wstring:
 	case CORBA_tk_string:
