@@ -181,7 +181,7 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 
 	    *val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_STRUCT);
 
-	    utc = ORBit_get_union_tag(tc, val);
+	    utc = ORBit_get_union_tag(tc, val, TRUE);
 
 	    *val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_STRUCT);
 
@@ -276,18 +276,18 @@ static glong ORBit_get_union_switch(CORBA_TypeCode tc, gpointer *val, gboolean u
     case CORBA_tk_long:
     case CORBA_tk_enum:
 	retval = *(CORBA_long *)*val;
-	if(update) val += sizeof(CORBA_long);
+	if(update) *val += sizeof(CORBA_long);
 	break;
     case CORBA_tk_ushort:
     case CORBA_tk_short:
 	retval = *(CORBA_short *)*val;
-	if(update) val += sizeof(CORBA_short);
+	if(update) *val += sizeof(CORBA_short);
 	break;
     case CORBA_tk_char:
     case CORBA_tk_boolean:
     case CORBA_tk_octet:
 	retval = *(CORBA_octet *)*val;
-	if(update) val += sizeof(CORBA_char);
+	if(update) *val += sizeof(CORBA_char);
 	break;
     case CORBA_tk_alias:
 	return ORBit_get_union_switch(tc->subtypes[0], val, update);
@@ -303,17 +303,19 @@ static glong ORBit_get_union_switch(CORBA_TypeCode tc, gpointer *val, gboolean u
    sole purpose of finding out which CORBA_TypeCode a union discriminator value
    indicates.
 
+   If {update} is TRUE, {*val} will be advanced by the native size
+   of the descriminator type.
+
    Hairy stuff.
 */
 CORBA_TypeCode
-ORBit_get_union_tag(CORBA_TypeCode union_tc,
-		    gpointer *val)
+ORBit_get_union_tag(CORBA_TypeCode union_tc, gpointer *val, gboolean update)
 {
     glong discrim_val, case_val;
     int i;
     CORBA_TypeCode retval = CORBA_OBJECT_NIL;
 
-    discrim_val = ORBit_get_union_switch(union_tc->discriminator, val, TRUE);
+    discrim_val = ORBit_get_union_switch(union_tc->discriminator, val, update);
 
     for(i = 0; i < union_tc->sub_parts; i++) {
 	if(i == union_tc->default_index)
@@ -602,7 +604,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	    }
 	    *val = ALIGN_ADDRESS(*val, n);
 	    ORBit_demarshal_value(buf, val,
-				  ORBit_get_union_tag(tc, discrimptr),
+				  ORBit_get_union_tag(tc, &discrimptr, FALSE),
 				  dup_strings, orb);
 	}
 	break;
@@ -616,7 +618,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	else
 	    *(char **)*val = buf->cur;
 	*val = ((guchar *)*val) + sizeof(CORBA_char *);
-	buf->cur += i;
+	buf->cur = (gpointer)((char *)buf->cur + i);
 	break;
     case CORBA_tk_sequence:
 	{
@@ -704,7 +706,7 @@ ORBit_demarshal_any(GIOPRecvBuffer *buf, CORBA_any *retval,
     ORBit_demarshal_value(buf, &val, retval->_type, dup_strings, orb);
 }
 
-static void
+void
 _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 {
     CORBA_long i;
@@ -818,15 +820,20 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 	break;
     case CORBA_tk_union:
 	{
-	    CORBA_TypeCode utc = ORBit_get_union_tag(tc, val);
+	    CORBA_TypeCode utc = ORBit_get_union_tag(tc, val, FALSE);
+	    gint	union_align = ORBit_find_alignment(tc);
+	    size_t	union_size = ORBit_gather_alloc_info(tc);
 
-	    *val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_STRUCT);
-	    *newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_STRUCT);
-
-	    _ORBit_copy_value(val, newval, tc->discriminator);
-	    *val = ALIGN_ADDRESS(*val, ORBit_find_alignment(tc));
-	    *newval = ALIGN_ADDRESS(*newval, ORBit_find_alignment(tc));
-	    _ORBit_copy_value(val, newval, utc);
+	    /* need to advance val,newval by size of union, not just
+	     * current tagged field within it */
+	    pval1 = *val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_STRUCT);
+	    pval2 = *newval = ALIGN_ADDRESS(*newval, ALIGNOF_CORBA_STRUCT);
+	    _ORBit_copy_value(&pval1, &pval2, tc->discriminator);
+	    pval1 = ALIGN_ADDRESS(pval1, union_align);
+	    pval2 = ALIGN_ADDRESS(pval2, union_align);
+	    _ORBit_copy_value(&pval1, &pval2, utc);
+	    *val = ((guchar *)*val) + union_size;
+	    *newval = ((guchar *)*newval) + union_size;
 	}
 	break;
     case CORBA_tk_wstring:
@@ -847,11 +854,13 @@ _ORBit_copy_value(gpointer *val, gpointer *newval, CORBA_TypeCode tc)
 				MAX(MAX(ALIGNOF_CORBA_LONG,
 					ALIGNOF_CORBA_STRUCT),
 				    ALIGNOF_CORBA_POINTER));
-	*(CORBA_Principal *)*newval = *(CORBA_Principal *)*val;
 	((CORBA_Principal *)*newval)->_release = CORBA_TRUE;
+	((CORBA_Principal *)*newval)->_length =
+	     ((CORBA_Principal *)*newval)->_maximum =
+					 ((CORBA_Principal *)*val)->_length;
 	((CORBA_Principal *)*newval)->_buffer = pval2 =
 	    ORBit_demarshal_allocate_mem(tc->subtypes[0],
-					 ((CORBA_Principal *)*newval)->_length);
+					 ((CORBA_Principal *)*val)->_length);
 	pval1 = ((CORBA_Principal *)*val)->_buffer;
 	
 	for(i = 0; i < ((CORBA_Principal *)*newval)->_length; i++) {
