@@ -115,18 +115,19 @@ void CORBA_exception_set(CORBA_Environment *ev, CORBA_exception_type major,
 	ev->_params=param;
 }
 
-void CORBA_exception_set_system(CORBA_Environment *ev, CORBA_unsigned_long minor, CORBA_completion_status completed)
+void CORBA_exception_set_system(CORBA_Environment *ev, CORBA_unsigned_long ex_value,
+				CORBA_completion_status completed)
 {
 	CORBA_SystemException *new;
 
 	new=ORBit_alloc(sizeof(CORBA_SystemException), NULL, NULL);
 	if(new!=NULL) {
-		new->minor=minor;
+		new->minor=0;
 		new->completed=completed;
 
 		/* XXX what should the repo ID be? */
 		CORBA_exception_set(ev, CORBA_SYSTEM_EXCEPTION,
-				    (gchar *)exception_table[minor].repo_id,
+				    exception_table[ex_value].repo_id,
 				    new);
 	}
 }
@@ -205,7 +206,7 @@ void ORBit_handle_exception(GIOPRecvBuffer *rb, CORBA_Environment *ev,
 			    CORBA_ORB orb)
 {
 	CORBA_SystemException *new;
-	CORBA_unsigned_long len, minor, completion_status;
+	CORBA_unsigned_long len, completion_status;
 	CORBA_char *my_repoid;
 
 	g_return_if_fail(GIOP_MESSAGE_BUFFER(rb)->message_header.message_type == GIOP_REPLY);
@@ -224,15 +225,13 @@ void ORBit_handle_exception(GIOPRecvBuffer *rb, CORBA_Environment *ev,
 		my_repoid = NULL;
 
 	if(rb->message.u.reply.reply_status == CORBA_SYSTEM_EXCEPTION) {
-		ev->_major = CORBA_SYSTEM_EXCEPTION;
-		rb->cur = ALIGN_ADDRESS(rb->cur, sizeof(minor));
-		rb->decoder(&minor, rb->cur, sizeof(len));
-		rb->cur = ((guchar *)rb->cur) + sizeof(len);
+		CORBA_unsigned_long minor;
 
-		for(minor = 1; exception_table[minor].repo_id; minor++) {
-			if(!strcmp(exception_table[minor].repo_id, my_repoid))
-				break;
-		}
+		ev->_major = CORBA_SYSTEM_EXCEPTION;
+
+		rb->cur = ALIGN_ADDRESS(rb->cur, sizeof(minor));
+		rb->decoder(&minor, rb->cur, sizeof(minor));
+		rb->cur = ((guchar *)rb->cur) + sizeof(minor);
 
 		rb->cur = ALIGN_ADDRESS(rb->cur, sizeof(completion_status));
 		rb->decoder(&completion_status, rb->cur, sizeof(completion_status));
@@ -244,10 +243,10 @@ void ORBit_handle_exception(GIOPRecvBuffer *rb, CORBA_Environment *ev,
 			new->minor=minor;
 			new->completed=completion_status;
 			
-			/* XXX what should the repo ID be? */
-			CORBA_exception_set(ev, CORBA_SYSTEM_EXCEPTION,
-					    (gchar *)exception_table[new->minor].repo_id,
-					    new);
+		/* XXX what should the repo ID be? */
+		CORBA_exception_set(ev, CORBA_SYSTEM_EXCEPTION,
+				    my_repoid,
+				    new);
 		}
 	} else if(rb->message.u.reply.reply_status == CORBA_USER_EXCEPTION) {
 		int i;
@@ -290,7 +289,9 @@ ORBit_send_system_exception(GIOPSendBuffer *send_buffer,
 	minor = se->minor;
 	completion_status = se->completed;
 
-	ENCODER_CALL(CORBA_char, exception_table[minor].repo_id);
+	ENCODER_CALL(CORBA_char, ev->_repo_id);
+	giop_send_buffer_append_mem_indirect_a(send_buffer, &minor,
+					       sizeof(minor));
 	giop_send_buffer_append_mem_indirect_a(send_buffer,
 					       &completion_status,
 					       sizeof(completion_status));
