@@ -381,7 +381,7 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 	PortableServer_ServantLocator_Cookie cookie;
 	ORBit_POAObject *obj_impl = NULL, tmp_obj_impl;
 	ORBitSkeleton skel;
-	gpointer imp;
+	gpointer imp = NULL;
 	CORBA_Environment ev;
 	GIOPSendBuffer *send_buffer;
 	guchar *opname;
@@ -483,20 +483,17 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 
 	skel = ORBIT_OBJECT_KEY(servant->_private)->class_info->relay_call(servant, recv_buffer, &imp);
 	
-	if(!skel && opname[0] == '_') {
-		guchar *opname = recv_buffer->message.u.request.operation;
-
-		if(!strcmp(opname, "_is_a"))
+	if(!skel) {
+		if (opname[0] == '_' && strcmp(opname + 1, "is_a") == 0) {
 			skel = (gpointer)&ORBit_impl_CORBA_Object_is_a;
+		}
+		else {
+			CORBA_exception_set_system(&ev, ex_CORBA_BAD_OPERATION,
+						   CORBA_COMPLETED_NO);
+			goto errout;
+		}
 	}
-
-	if (!skel) {
-		CORBA_exception_set_system(&ev, ex_CORBA_BAD_OPERATION,
-					   CORBA_COMPLETED_NO);
-		goto errout;
-	}
-
-	if (!imp) {
+	else if (!imp) {
 		CORBA_exception_set_system(&ev, ex_CORBA_NO_IMPLEMENT,
 					   CORBA_COMPLETED_NO);
 		goto errout;
