@@ -18,14 +18,16 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: orbit-idl-backends.c,v 1.5 1999/07/22 13:58:02 sopwith Exp $
+    $Id: orbit-idl-backends.c,v 1.7 1999/12/23 16:57:00 sopwith Exp $
 
 ***************************************************************************/
 
+#include "config.h"
 #include "orbit-idl2.h"
 #include "backends/c/orbit-idl-c-backend.h"
 #include <dirent.h>
 #include <gmodule.h>
+#include <string.h>
 
 static OIDL_Backend_Info orbit_idl_builtin_backends[] = {
   {"c", &orbit_idl_output_c},
