@@ -18,7 +18,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: orbit-idl-main.c,v 1.4 1999/06/25 13:58:34 wilhelmi Exp $
+    $Id: orbit-idl-main.c,v 1.4.4.1 2000/03/09 17:16:49 sopwith Exp $
 
 ***************************************************************************/
 
@@ -67,7 +67,8 @@ cl_cpp_callback(poptContext con, enum poptCallbackReason reason,
 {
   g_assert(opt!=NULL);
 
-  if(opt->shortName=='D')
+  if(!strcmp(opt->longName, "D")
+     || !strcmp(opt->longName, "define"))
     g_string_append(cl_cpp_args, "-D");
   else if(opt->shortName=='I')
     g_string_append(cl_cpp_args, "-I");
